/**
 * \file utils.c
 *
 *  20.11.2012  psa  Various useful non ansi-c functions
 */


#include "main.h"
#include "utils.h"
#include <ctype.h>
 
static u08 ascii[16] = {'0','1','2','3','4','5','6','7','8','9','A','B','C','D','E','F'};

//---------------------------------------------------------------------
// Convert a string to upper case.
// The strupr() function will convert a string to upper case. 
// Only the lower case alphabetic characters [a .. z] are converted.
// Non-alphabetic characters will not be changed.
// Returns:
// The strupr() function returns a pointer to the converted string.
// The pointer is the same as that passed in since the operation is
// perform in place.
//---------------------------------------------------------------------
char *strupr(char *string)
{
  char *s;
  if (string)
  {
    for (s = string; *s; ++s)
    *s = toupper((int)*s);
  }
 return string;
} 

//---------------------------------------------------------------------
// The strlwr() function will convert a string to lower case. 
// Only the upper case alphabetic characters [A .. Z] are converted.
// Non-alphabetic characters will not be changed.
// Returns: 
// The strlwr() function returns a pointer to the converted string.
// The pointer is the same as that passed in since the operation is
// perform in place.
//---------------------------------------------------------------------
char *strlwr(char *string)
{
 char *s;
 if (string)
 {
   for (s = string; *s; ++s)
   *s = tolower((int)*s);
 }
 return string;
}

u08* _int2hex(u08 *intval, u08 *hexstr, u08 n)
//---------------------------------------------------------------------
// Converts an integer with a size of [n] bytes to a hex-digit string.
// The string will be filled up with leading zeros to his length.
//---------------------------------------------------------------------
{
  u08 e=0,c;
  while(n--)
  {
    c=intval[n];
    hexstr[e++]=ascii[(c >> 0x04)];
    hexstr[e++]=ascii[(c &  0x0F)];
  }
  hexstr[e]=0x00;
  return(hexstr);
}

u08* _bin2hex(u08 *intval, u08 *hexstr, u08 n)
//---------------------------------------------------------------------
// Converts a sequence of [n] bytes to a hex-digit string.
//---------------------------------------------------------------------
{
  u08 e=0,i=0,c;
  while(i<n)
  {
    c=intval[i++];
    hexstr[e++]=ascii[(c >> 0x04)];
    hexstr[e++]=ascii[(c &  0x0F)];
  }
  hexstr[e]=0x00;
  return(hexstr);
}


void _hex2int(char *hexstr, u08 *intval, u08 n)
//---------------------------------------------------------------------
// Converts a hex-digit string to an integer with a size of [n] bytes.
// If the string doesn't contain enought hex-digits, the string will
// be modified by right alignment and insertion of leading zeros.
//---------------------------------------------------------------------
{
  u08 chr,v,i,p;
  //-------------------------------------------------------------------
  // Convert string to binary integer
  //-------------------------------------------------------------------
  i=strlen(hexstr);
  p=0;
  while(p<n)                           // loop for each byte
  {
    //-----------------------------------------------------------------
    // lower nibble
    //-----------------------------------------------------------------
    v=0;
    if (i)
    {
      chr=hexstr[--i];
      if (chr!='X')                     // skip 'x' of  0xABCD number
      {
        if (chr<'A') v = (chr-'0');     // if '0'..'9' => value = (chr + 0 - '0')
        else v = (chr-('A'-10));        // if 'A'..'F' => value = (chr +10 - 'A')
      }
    }
    //-----------------------------------------------------------------
    // upper nibble
    //-----------------------------------------------------------------
    if (i)
    {
      chr=hexstr[--i];
      if (chr!='X')                     // skip 'x' of  0xABCD number
      {
        if (chr<'A') v |= (chr-'0')<<4; // if '0'..'9' => value |= (chr + 0 - '0')
        else v |= (chr-('A'-10))<<4;    // if 'A'..'F' => value |= (chr +10 - 'A')
      }
    }
    intval[p++]=v;
  }
}

u08 _hex2bin(char *hexstr, u08 *intval, u08 n)
//---------------------------------------------------------------------
// Converts a hex-digit string to a sequenze of [n] bytes.
// if [n] is zero, the sequenze will be limited by the stringlength
// The length of the sequence is provided as return value.
//---------------------------------------------------------------------
{
  u08 chr, hex=0, e=0, i=0;
  if (n==0) n=124;                 // if n=0 then set maximum stringlength to 2*124 char
  if (strlen(hexstr)&1)            // if odd number of nibbels insert leading '0'
  { 
    chr=hexstr[i++];               // get first char (low nibble)
    if (chr<'A') hex = (chr-'0');  // if '0'..'9' => value = (chr + 0 - '0')
    else hex = (chr-('A'-10));     // if 'a'..'f' => value = (chr +10 - 'A')
    intval[e++]=hex;               // store binary byte value
  }
  chr=hexstr[i++];                 // get next char (high nibble)
  while((e<n) && (chr))            // while not stringend
  {
    if (chr<'A') hex = (chr-'0');  // if '0'..'9' => value = (chr + 0 - '0')
    else hex = (chr-('A'-10));     // if 'a'..'f' => value = (chr +10 - 'A')
    hex = hex<<4;                  // nibble shift (char = char * 16)
    chr=hexstr[i++];               // get next char (low nibble)
    if (chr<'A') hex |= (chr-'0'); // if '0'..'9' => value = (chr + 0 - '0')
    else hex |= (chr-('A'-10));    // if 'a'..'f' => value = (chr +10 - 'A')
    intval[e++]=hex;               // store binary byte value
    chr=hexstr[i++];               // get next char (next high nibble)
  }
  return(e);
}


/**
 \ingroup avr_stdlib
   
   \brief Convert an integer to a string.

   The function itoa() converts the integer value from \c val into an
   ASCII representation that will be stored under \c s.  The caller
   is responsible for providing sufficient storage in \c s.

   \note The minimal size of the buffer \c s depends on the choice of
   radix. For example, if the radix is 2 (binary), you need to supply a buffer
   with a minimal length of 8 * sizeof (int) + 1 characters, i.e. one
   character for each bit plus one for the string terminator. Using a larger
   radix will require a smaller minimal buffer size.

   \warning If the buffer is too small, you risk a buffer overflow.

   Conversion is done using the \c radix as base, which may be a
   number between 2 (binary conversion) and up to 36.  If \c radix
   is greater than 10, the next digit after \c '9' will be the letter
   \c 'a'.
    
    If radix is 10 and val is negative, a minus sign will be prepended.

   The itoa() function returns the pointer passed as \c s.
*/
char *itoa(s32 value, char *string, int radix)
{
  char tmp[33];
  char *tp = tmp;
  s32 i;
  u32 v;
  int sign;
  char *sp;
  if (radix > 36 || radix <= 1)
  {
    return 0;
  }
  sign = (radix == 10 && value < 0);
  if (sign)
  {
    v = -value;
  }
  else
  {
    v = (u32)value;
  }
  while (v || tp == tmp)
  {
    i = v % radix;
    v = v / radix;
    if (i < 10)
    {
      *tp++ = i+'0';
    }
    else
    {
      *tp++ = i + 'A' - 10;
    }
  }
  if (string == 0)
  {
    string = (char *)malloc((tp-tmp)+sign+1);
  }
  sp = string;
  if (sign)
  {
    *sp++ = '-';
  }
  while (tp > tmp)
  {
    *sp++ = *--tp;
  }
  *sp = 0;
  return string;
}

/**
 \ingroup avr_stdlib

   \brief Convert an unsigned integer to a string.

   The function utoa() converts the unsigned integer value from \c val into an
   ASCII representation that will be stored under \c s.  The caller
   is responsible for providing sufficient storage in \c s.

   \note The minimal size of the buffer \c s depends on the choice of
   radix. For example, if the radix is 2 (binary), you need to supply a buffer
   with a minimal length of 8 * sizeof (unsigned int) + 1 characters, i.e. one
   character for each bit plus one for the string terminator. Using a larger
   radix will require a smaller minimal buffer size.

   \warning If the buffer is too small, you risk a buffer overflow.

   Conversion is done using the \c radix as base, which may be a
   number between 2 (binary conversion) and up to 36.  If \c radix
   is greater than 10, the next digit after \c '9' will be the letter
   \c 'a'.

   The utoa() function returns the pointer passed as \c s.
*/
char *utoa(u32 value, char *string, int radix)
{
  char tmp[33];
  char *tp = tmp;
  s32 i;
  u32 v = value;
  char *sp;
  if (radix > 36 || radix <= 1)
  {
    return 0;
  }
  while (v || tp == tmp)
  {
    i = v % radix;
    v = v / radix;
    if (i < 10)
    {
      *tp++ = i+'0';
    }
    else
    {
      *tp++ = i + 'A' - 10;
    }
  }
  if (string == 0)
  {
    string = (char *)malloc((tp-tmp)+1);
  }
  sp = string;
  while (tp > tmp)
  {
    *sp++ = *--tp;
  }
  *sp = 0;
  return string;
}
